@@ -23,6 +23,13 @@ PLUGIN(NAME("wdc", "Western Digital vendor specific extensions"),
 		ENTRY("get-drive-status", "WDC Get Drive Status", wdc_drive_status)
 		ENTRY("clear-assert-dump", "WDC Clear Assert Dump", wdc_clear_assert_dump)
 		ENTRY("drive-resize", "WDC Drive Resize", wdc_drive_resize)
+		ENTRY("vs-fw-activate-history", "WDC Get FW Activate History", wdc_vs_fw_activate_history)
+		ENTRY("clear-fw-activate-history", "WDC Clear FW Activate History", wdc_clear_fw_activate_history)
+		ENTRY("vs-telemetry-controller-option", "WDC Enable/Disable Controller Initiated Telemetry Log", wdc_vs_telemetry_controller_option)
+		ENTRY("vs-error-reason-identifier", "WDC Telemetry Reason Identifier", wdc_reason_identifier)
+		ENTRY("log-page-directory", "WDC Get Log Page Directory", wdc_log_page_directory)
+		ENTRY("namespace-resize", "WDC NamespaceDrive Resize", wdc_namespace_resize)
+		ENTRY("vs-drive-info", "WDC Get Drive Info", wdc_vs_drive_info)
 	)
 );
 
