@@ -45,7 +45,7 @@ class TestNVMeGetMandatoryFeatures(TestNVMe):
         - Attributes:
               - feature_id_list : list of the mandatory features.
               - get_vector_list_cmd : vector list collection for 09h.
-              - vector_list : vector list to hold the interrupt vectors.
+              - vector_list_len : numer of the interrupt vectors.
     """
 
     def __init__(self):
@@ -58,9 +58,9 @@ class TestNVMeGetMandatoryFeatures(TestNVMe):
                               " cut -d : -f 1 | tr -d ' ' | tr '\n' ' '"
         proc = subprocess.Popen(get_vector_list_cmd,
                                 shell=True,
-                                stdout=subprocess.PIPE)
-        self.vector_list = []
-        self.vector_list = proc.stdout.read().strip().split(" ")
+                                stdout=subprocess.PIPE,
+                                encoding='utf-8')
+        self.vector_list_len = len(proc.stdout.read().strip().split(" "))
 
     def __del__(self):
         """ Post Section for TestNVMeGetMandatoryFeatures
@@ -77,13 +77,14 @@ class TestNVMeGetMandatoryFeatures(TestNVMe):
                 - None
         """
         if str(feature_id) == "0x09":
-            for vector in self.vector_list:
+            for vector in range(self.vector_list_len):
                 get_feat_cmd = "nvme get-feature " + self.ctrl + \
                                " --feature-id=" + str(feature_id) + \
                                " --cdw11=" + str(vector)
                 proc = subprocess.Popen(get_feat_cmd,
                                         shell=True,
-                                        stdout=subprocess.PIPE)
+                                        stdout=subprocess.PIPE,
+                                        encoding='utf-8')
                 feature_output = proc.communicate()[0]
                 print(feature_output)
                 assert_equal(proc.wait(), 0)
@@ -92,7 +93,8 @@ class TestNVMeGetMandatoryFeatures(TestNVMe):
                            " --feature-id=" + str(feature_id)
             proc = subprocess.Popen(get_feat_cmd,
                                     shell=True,
-                                    stdout=subprocess.PIPE)
+                                    stdout=subprocess.PIPE,
+                                    encoding='utf-8')
             feature_output = proc.communicate()[0]
             print(feature_output)
             assert_equal(proc.wait(), 0)
